#!/usr/bin/python

import json
import os.path
import sys

from efs.client.connection import Connection, RpcError
from efs.common.encryption import import_key

def read_session_file():
    home = os.path.expanduser("~") 
    session_file = home + "/.efs_session"
    if os.path.isfile(session_file):
        try:
            with open(session_file) as f:
                return json.load(f)
        except:
            print "Invalid session file. Exiting"
            sys.exit(1)
    else:
        print "Session file does not exist. Exiting"
        sys.exit(1)

def create_connection(session):
    try:
        conn = Connection(session["host"], session["port"])
        conn.token = session["token"]
        conn.user = session["user"]
        conn.session_key = session["session_key"]
        conn.private_key = import_key(session["private_key"])
        conn.public_key = import_key(session["public_key"])
        return conn
    except:
        print "Invalid session. Exiting"
        sys.exit(1)

def read_key(path):
    try:
        f = open(path)
        return import_key(f.read())
    except:
        raise

def handle_mkgroup(conn, argv):
    if len(argv) == 5:
        try:
            public_key = read_key(argv[3])
            private_key = read_key(argv[4])
        except:
            print >> sys.stderr, "Invalid keys"
            sys.exit(1)
        try:
            conn.mkgroup(argv[2], conn.user, public_key, private_key)
            print "Successfully created group " + argv[2]
        except RpcError:
            print >> sys.stderr, "Could not complete request"
            sys.exit(1)
    else:
        print >> sys.stderr, help_str

def handle_rmgroup(conn, argv):
    if len(argv) == 3:
        try:
            conn.rmgroup(argv[2])
            print "Successfully removed group: " + argv[2]
        except RpcError:
            print >> sys.stderr, "Could not complete request"
            sys.exit(1)
    else:
        print >> sys.stderr, help_str

def handle_adduser(conn, argv):
    if len(argv) == 4:
        try:
            conn.adduser(argv[2], argv[3])
            print "Successfully added user " + argv[3] + " to group " + argv[2]
        except RpcError:
            print >> sys.stderr, "Could not complete request"
    else:
        print >> sys.stderr, help_str

def handle_rmuser(conn, argv):
    if len(argv) == 4:
        try:
            conn.rmuser(argv[2], argv[3])
            print "Succcessfully removed user " + argv[3] + " from group " + argv[3]
        except:
            print >> sys.stderr, "Could not complete request"
    else:
        print >> sys.stderr, help_str

def handle_chpasswd(conn, argv):
    if len(argv) == 4:
        if argv[2] == argv[3]:
            try:
                conn.chpasswd(conn.user, argv[2])
                print "Successfully changed password of user "  + conn.user
            except:
                print >> sys.stderr, "Could not complete request"
    else:
         print >> sys.stderr, help_str

help_str = """ 
    Usage
    In order to use this script you should mount the file system beforehand
    user_management.py mkgroup group_name public_key_path private_key_path
    ------------------ adduser group_name user_name
    ------------------ rmgroup group_name
    ------------------ rmuser group_name user_name
    ------------------ chpasswd user new_passwd new_passwd_confirmation
    """

if __name__ == "__main__":
    if len(sys.argv) < 2:
        print >> sys.stderr, help_str
        sys.exit(1)
    
    session = read_session_file()
    conn = create_connection(session)
    if sys.argv[1] == "mkgroup":
        handle_mkgroup(conn, sys.argv)
    elif sys.argv[1] == "rmgroup":
        handle_rmgroup(conn, sys.argv)
    elif sys.argv[1] == "adduser":
        handle_adduser(conn, sys.argv)
    elif sys.argv[1] == "rmuser":
        handle_rmuser(conn, sys.argv)
    elif sys.argv[1] == "chpasswd":
        handle_chpasswd(conn, sys.argv)
    else:
        print >> sys.stderr, help_str
