import json
from efs.common.exceptions import InvalidConfigFileError
from efs.common.encryption import generate_pubkey, import_key, export_key

class ServerConfigFile:

	def __init__(self, config_file_path):
		"""
		Opens and parses the configuration file located at config_file_path.
		The file is a json file with key-value pairs representing configuration
		options and their value.
		"""

		#Set configurations from values stored in file
		try:
			config_file = open(config_file_path)
			json_config_file = json.load(config_file)
			config_file.close()
			self.root_dir = json_config_file["root_dir"]
			self.server_port = json_config_file["server_port"]
			self.mongo_host = json_config_file["mongo_host"]
			self.mongo_port = json_config_file["mongo_port"]
			if "server_public_key" not in json_config_file and "server_private_key" not in json_config_file:
				self.server_private_key = generate_pubkey()
				self.server_public_key = self.server_private_key.publickey()
				json_config_file["server_private_key"] = export_key(self.server_private_key)
				json_config_file["server_public_key"] = export_key(self.server_public_key)
				with open(config_file_path, "w") as f:
					json.dump(json_config_file, f, indent=4)
			else:
				self.server_public_key = import_key(json_config_file["server_public_key"])
				self.server_private_key = import_key(json_config_file["server_private_key"])
		except:
			raise InvalidConfigFileError


