from pymongo.errors import PyMongoError

from efs.server.exceptions import InvalidGroupError
from efs.server.exceptions import InvalidUserError
from efs.common.exceptions import InvalidKeyError
from efs.common.encryption import import_key

class KeyService(object):

	def __init__(self, mongo_client, server_public_key, server_private_key):
		self.mongo_client = mongo_client
		try:
			self.login_db = mongo_client.login
			self.users = self.login_db.users
			self.sessions = self.login_db.sessions
			self.groups = self.login_db.groups
			self.server_public_key = server_public_key
			self.server_private_key = server_private_key
		except:
			raise PyMongoError

	def get_key(self, key_id):
		(key_type, name) = self.parse_key_id(key_id)
		try:
			if key_type == "server":
				return self.get_server_private_key()
			elif key_type == "user":
				return self.get_user_public_key(name)
			elif key_type == "session":
				return self.get_session_key(name)
			else:
				raise InvalidKeyError
		except:
			raise InvalidKeyError

	def parse_key_id(self, key_id):
		key_parts = key_id.split(":")
		if len(key_parts) == 1:
			return (key_parts[0], None)
		elif len(key_parts) == 2:
			return (key_parts[0], key_parts[1])
		else:
			return (None, None)

	def get_server_public_key(self):
		return self.server_public_key

	def get_server_private_key(self):
		return self.server_private_key

	def get_user_public_key(self, user):
		user_record = self.users.find({"user":user})
		if user_record.count() == 1:
			return import_key(user_record[0]["public_key"])
		else:
			raise InvalidUserError

	def get_group_public_key(self, group):
		group_record = self.groups.find({"group":group})
		if group_record.count() == 1:
			return import_key(group_record[0]["public_key"])
		else:
			raise InvalidGroupError

	def get_group_private_key(self, group, user):
		group_record = self.groups.find({"group":group})
		if group_record.count() == 1:
			if user == group_record[0]["owner"]:
				return group_record[0]["owner_private_key"]
			if user in group_record[0]["members"]:
				return group_record[0]["members"][user]
			else:
				raise InvalidUserError
		else:
			raise InvalidGroupError

	def get_session_key(self, user):
		session_record = self.sessions.find({"user":user})
		if session_record.count() == 1:
			return session_record[0]["session_key"]
		else:
			raise InvalidUserError



