#!/usr/bin/python

import sys

from pymongo import MongoClient
from pymongo.errors import ConnectionFailure

from efs.common.exceptions import InvalidConfigFileError
from efs.server.authorization import AuthorizationService
from efs.server.config import ServerConfigFile
from efs.server.login import LoginService
from efs.server.keys import KeyService
from efs.server.structure import StructService
from efs.server.server import EFSServer
from efs.server import fs

if __name__ == "__main__":
	#Parse server config file
	print "Parsing configuration file"
	config_file_path = sys.argv[1]
	try:
		config_file = ServerConfigFile(config_file_path)
	except InvalidConfigFileError:
		print "Could not parse configuration file. Exiting"
		sys.exit(1)
	#start database
	print "Connecting to database"
	try:
		mongo_client = MongoClient(config_file.mongo_host, config_file.mongo_port)
	except ConnectionFailure:
		print "Could not connect to database. Exiting"
		sys.exit(1)
	#Setup login service
	login_service = LoginService(mongo_client)
	#Setup key service
	key_service = KeyService(mongo_client, config_file.server_public_key, config_file.server_private_key)
	#Setup authorization system
	authorization_service = AuthorizationService(mongo_client)
	#Setup struct service
	struct_service = StructService(mongo_client)
	#Setup filesystem
	fs.init(mongo_client)
	#Start HTTP server
	print "Starting HTTP server"
	try:
		efs_server = EFSServer(config_file.server_port, login_service, authorization_service,
		 	key_service, struct_service)
		efs_server.start()
	except:
		print "Could not start the server. Exiting"
		sys.exit(1)
	


